# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine EndpointCertificate API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

from builtins import *

from past.builtins import basestring

from ...restsession import RestSession
from ...utils import (
    check_type,
    dict_from_items_with_values,
    apply_path_params,
    dict_of_str,
)
from ...pagination import get_next_page


class EndpointCertificate(object):
    """Identity Services Engine EndpointCertificate API (version: 3.1.0).

    Wraps the Identity Services Engine EndpointCertificate
    API and exposes the API as native Python
    methods that return native Python objects.

    | Endpoint Certificate API allows the client to create endpoint certificates signed by the Cisco ISE Internal CA. This API can takes in certificate request details, create a RSA key pair, create a certificate and return the resulting key pair and certificate as a ZIP file. ZIP files are returned as an octet stream.

    **Revision History**

    +----------------+----------------------+-----------------------+---------------------------+
    | **Revision #** | **Resource Version** | **Cisco ISE Version** | **Description**           |
    +----------------+----------------------+-----------------------+---------------------------+
    | 0              | 1.0                  | 2.0                   | Initial Cisco ISE Version |
    +----------------+----------------------+-----------------------+---------------------------+

    |

    **Resource Definition**

    +--------------------+-------------+--------------+-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+---------------------------+
    | **Attribute**      | **Type**    | **Required** | **Description**                                                                                                                                                                                                 | **Example Values**        |
    +--------------------+-------------+--------------+-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+---------------------------+
    | name               | String      | Yes          | Resource Name                                                                                                                                                                                                   |                           |
    +--------------------+-------------+--------------+-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+---------------------------+
    | id                 | String      | No           | Resource UUID, mandatory for update                                                                                                                                                                             |                           |
    +--------------------+-------------+--------------+-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+---------------------------+
    | description        | String      | No           |                                                                                                                                                                                                                 |                           |
    +--------------------+-------------+--------------+-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+---------------------------+
    | certTemplateName   | String      | Yes          | Name of an Internal CA template                                                                                                                                                                                 | Certificate_Template_Name |
    +--------------------+-------------+--------------+-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+---------------------------+
    | format             | Enum        | Yes          | Allowed values: PKCS12, PKCS12_CHAIN, PKCS8, PKCS8_CHAIN                                                                                                                                                        | PKCS8                     |
    +--------------------+-------------+--------------+-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+---------------------------+
    | password           | String      | Yes          | Protects the private key. Must have more than 8 characters, less than 15 characters, at least one upper case letter, at least one lower case letter, at least one digit, and can only contain [A-Z][a-z][0-9]_# | Password_123              |
    +--------------------+-------------+--------------+-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+---------------------------+
    | certificateRequest | Map         | Yes          | Key value map. Must have CN and SAN entries                                                                                                                                                                     |                           |
    +--------------------+-------------+--------------+-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+---------------------------+
    | - CN               | String      | Yes          | Matches the requester's User Name, unless the Requester is an ERS Admin. ERS Admins are allowed to create requests for any CN                                                                                   | userName [or] machineName |
    +--------------------+-------------+--------------+-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+---------------------------+
    | - SAN              | String      | Yes          | Valid MAC Address, delimited by '-'                                                                                                                                                                             | 11-22-33-44-55-66         |
    +--------------------+-------------+--------------+-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+---------------------------+

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new EndpointCertificate
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(EndpointCertificate, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def create_endpoint_certificate(self,
                                    cert_template_name=None,
                                    certificate_request=None,
                                    format=None,
                                    password=None,
                                    dirpath=None,
                                    save_file=None,
                                    headers=None,
                                    payload=None,
                                    active_validation=True,
                                    **query_parameters):
        """This API allows the client to create an endpoint certificate.

        Args:
            cert_template_name(string): Name of an Internal CA
                template, property of the request body.
            certificate_request(object): Key value map. Must have CN
                and SAN entries, property of the request
                body.
            format(string): Allowed values: PKCS12, PKCS12_CHAIN,
                PKCS8, PKCS8_CHAIN, property of the
                request body.
            password(string): Protects the private key. Must have
                more than 8 characters, less than 15
                characters, at least one upper case
                letter, at least one lower case letter,
                at least one digit, and can only contain
                [A-Z][a-z][0-9]_#, property of the
                request body.
            dirpath(basestring): Directory absolute path. Defaults to
                os.getcwd().
            save_file(bool): Enable or disable automatic file creation of
                raw response.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:
            urllib3.response.HTTPResponse: HTTP Response container. For more
            information check the `urlib3 documentation <https://urllib3.readthedocs.io/en/latest/reference/urllib3.response.html>`_

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
            DownloadFailure: If was not able to download the raw
            response to a file.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           basestring)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'certTemplateName':
                    cert_template_name,
                'format':
                    format,
                'password':
                    password,
                'certificateRequest':
                    certificate_request,
            }
            _payload = {
                'ERSEndPointCert': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_e27d5df9cbe5b29a7e16bb7c877a4ce_v3_1_0')\
                .validate(_payload)

        e_url = ('/ers/config/endpointcert/certRequest')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              stream=True, dirpath=dirpath, save_file=save_file,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              stream=True, dirpath=dirpath, save_file=save_file,
                                              **request_params)

        return self._object_factory('bpm_e27d5df9cbe5b29a7e16bb7c877a4ce_v3_1_0', _api_response)

    def create(self,
               cert_template_name=None,
               certificate_request=None,
               format=None,
               password=None,
               dirpath=None,
               save_file=None,
               headers=None,
               payload=None,
               active_validation=True,
               **query_parameters):
        """Alias for `create_endpoint_certificate <#ciscoisesdk.
        api.v3_1_0.endpoint_certificate.
        EndpointCertificate.create_endpoint_certificate>`_
        """
        return self.create_endpoint_certificate(
            cert_template_name=cert_template_name,
            certificate_request=certificate_request,
            format=format,
            password=password,
            dirpath=dirpath,
            save_file=save_file,
            payload=payload,
            active_validation=active_validation,
            headers=headers,
            **query_parameters
        )

    def get_version(self,
                    headers=None,
                    **query_parameters):
        """This API helps to retrieve the version information related to
        the endpoint certificate.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/ers/config/endpointcert/versioninfo')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_c5c84028d8f5c078d8ab37553812d39_v3_1_0', _api_response)



# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine NodeDetails API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

from builtins import *

from past.builtins import basestring

from ...restsession import RestSession
from ...utils import (
    check_type,
    dict_from_items_with_values,
    apply_path_params,
    dict_of_str,
)
from ...pagination import get_next_page


class NodeDetails(object):
    """Identity Services Engine NodeDetails API (version: 3.1.0).

    Wraps the Identity Services Engine NodeDetails
    API and exposes the API as native Python
    methods that return native Python objects.

    | Node Details API allows the client to get deployment node details, get all nodes and filter nodes based on NodeServiceTypes.

    **Revision History**

    +----------------+----------------------+-----------------------+---------------------------------------------------+---------------------------+----------------------------------------------------------------------------------------+
    | **Revision #** | **Resource Version** | **Cisco ISE Version** | **Description**                                   | **Revision Modification** | **Revision Modification**                                                              |
    +----------------+----------------------+-----------------------+---------------------------------------------------+---------------------------+----------------------------------------------------------------------------------------+
    |                |                      |                       |                                                   | **Attribute**             | **Description**                                                                        |
    +----------------+----------------------+-----------------------+---------------------------------------------------+---------------------------+----------------------------------------------------------------------------------------+
    | 0              | 1.0                  | 2.2                   | Initial Cisco ISE Version                         |                           |                                                                                        |
    +----------------+----------------------+-----------------------+---------------------------------------------------+---------------------------+----------------------------------------------------------------------------------------+
    | 1              | 1.1                  | 2.4                   | Support Node Service Types as part of Node Object | NodeServiceTypes          | Added String Field NodeServiceTypes. String represents services configured on the Node |
    +----------------+----------------------+-----------------------+---------------------------------------------------+---------------------------+----------------------------------------------------------------------------------------+
    | 2              | 1.2                  | 2.7                   | Support Node FQDN as part of Node Object          | fqdn                      | Added String Field FQDN. String represents FQDN of the Node                            |
    +----------------+----------------------+-----------------------+---------------------------------------------------+---------------------------+----------------------------------------------------------------------------------------+
    | 3              | 1.3                  | 3.0                   | Support SXP IP Address as part of Node Details    | sxpIpAddress              | Added String Field sxpIpAddress. String represents SXP IP Address of the Node          |
    +----------------+----------------------+-----------------------+---------------------------------------------------+---------------------------+----------------------------------------------------------------------------------------+

    |

    **Resource Definition**

    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | **Attribute**    | **Type** | **Required** | **Description**                     | **Example Values**                   |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | name             | String   | Yes          | Resource Name                       | TestNode                             |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | id               | String   | No           | Resource UUID, mandatory for update | ab6deded-fcc2-47ff-8577-0014737c8fcf |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | description      | String   | No           |                                     |                                      |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | gateWay          | String   | No           |                                     | 10.0.10.11                           |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | displayName      | String   | No           |                                     | TestNode                             |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | inDeployment     | Boolean  | No           |                                     | false                                |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | ipAddresses      | String   | No           |                                     | [ "10.0.0.1", "10.0.0.2" ]           |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | otherPapFqdn     | String   | No           |                                     | OtherTestNode                        |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | ipAddress        | String   | No           |                                     | 10.0.0.3                             |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | sxpIpAddress     | String   | No           |                                     | 10.10.10.1                           |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | nodeServiceTypes | String   | No           |                                     | DEVICE ADMIN                         |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | fqdn             | String   | No           |                                     | test.ers.cisco.com                   |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | primaryPapNode   | Boolean  | No           |                                     | true                                 |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | papNode          | Boolean  | No           |                                     | true                                 |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+
    | pxGridNode       | Boolean  | No           |                                     | true                                 |
    +------------------+----------+--------------+-------------------------------------+--------------------------------------+

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new NodeDetails
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(NodeDetails, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def get_node_detail_by_name(self,
                                name,
                                headers=None,
                                **query_parameters):
        """This API allows the client to get node details by name.

        Args:
            name(basestring): name path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           basestring)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(name, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'name': name,
        }

        e_url = ('/ers/config/node/name/{name}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_ab48268c76aa598788a5ebc370226f3a_v3_1_0', _api_response)

    def get_by_name(self,
                    name,
                    headers=None,
                    **query_parameters):
        """Alias for `get_node_detail_by_name <#ciscoisesdk.
        api.v3_1_0.node_details.
        NodeDetails.get_node_detail_by_name>`_
        """
        return self.get_node_detail_by_name(
            name=name,
            headers=headers,
            **query_parameters
        )

    def get_node_detail_by_id(self,
                              id,
                              headers=None,
                              **query_parameters):
        """This API allows the client to get node details by ID.

        Args:
            id(basestring): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           basestring)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(id, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        e_url = ('/ers/config/node/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_bb2e9d6651c7bf18c1b60ff7eb14_v3_1_0', _api_response)

    def get_by_id(self,
                  id,
                  headers=None,
                  **query_parameters):
        """Alias for `get_node_detail_by_id <#ciscoisesdk.
        api.v3_1_0.node_details.
        NodeDetails.get_node_detail_by_id>`_
        """
        return self.get_node_detail_by_id(
            id=id,
            headers=headers,
            **query_parameters
        )

    def get_node_details(self,
                         filter=None,
                         filter_type=None,
                         page=None,
                         size=None,
                         headers=None,
                         **query_parameters):
        """This API allows the client to get all the node details.
        Filter:   [nodeservicetypes]   To search resources by
        using  toDate  column,follow the format:   DD-MON-YY
        (Example:13-SEP-18)     Day or Year:GET
        /ers/config/guestuser/?filter=toDate.CONTAINS.13
        Month:GET
        /ers/config/guestuser/?filter=toDate.CONTAINS.SEP
        Date:GET
        /ers/config/guestuser/?filter=toDate.CONTAINS.13-SEP-18
        .

        Args:
            page(int): page query parameter. Page number.
            size(int): size query parameter. Number of objects
                returned per page.
            filter(basestring, list, set, tuple): filter query
                parameter.               **Simple
                filtering** should be available through
                the filter query string parameter. The
                structure of a filter is a triplet of
                field operator and value separated with
                dots. More than one filter can be sent.
                The logical operator common to ALL
                filter criteria will be by default AND,
                and can be changed by using the
                "filterType=or" query string parameter.
                Each resource Data model description
                should specify if an attribute is a
                filtered field.              (Operator:
                Description),
                (EQ: Equals),               (NEQ: Not
                Equals),               (GT: Greater
                Than),               (LT: Less Then),
                (STARTSW: Starts With),
                (NSTARTSW: Not Starts With),
                (ENDSW: Ends With),
                (NENDSW: Not Ends With),
                (CONTAINS: Contains),
                (NCONTAINS: Not Contains),
                .
            filter_type(basestring): filterType query parameter. The
                logical operator common to ALL filter
                criteria will be by default AND, and can
                be changed by using the parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           basestring)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(page, (int, basestring, list))
        check_type(size, (int, basestring, list))
        check_type(filter, (basestring, list, set, tuple))
        check_type(filter_type, basestring)

        _params = {
            'page':
                page,
            'size':
                size,
            'filter':
                filter,
            'filterType':
                filter_type,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/ers/config/node')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_d9b8599f55fc4a1bd9d6ac02619eb_v3_1_0', _api_response)

    def get_all(self,
                filter=None,
                filter_type=None,
                page=None,
                size=None,
                headers=None,
                **query_parameters):
        """Alias for `get_node_details <#ciscoisesdk.
        api.v3_1_0.node_details.
        NodeDetails.get_node_details>`_
        """
        return self.get_node_details(
            filter=filter,
            filter_type=filter_type,
            page=page,
            size=size,
            headers=headers,
            **query_parameters
        )

    def get_node_details_generator(self,
                                   filter=None,
                                   filter_type=None,
                                   page=None,
                                   size=None,
                                   headers=None,
                                   **query_parameters):
        """This API allows the client to get all the node details.
        Filter:   [nodeservicetypes]   To search resources by
        using  toDate  column,follow the format:   DD-MON-YY
        (Example:13-SEP-18)     Day or Year:GET
        /ers/config/guestuser/?filter=toDate.CONTAINS.13
        Month:GET
        /ers/config/guestuser/?filter=toDate.CONTAINS.SEP
        Date:GET
        /ers/config/guestuser/?filter=toDate.CONTAINS.13-SEP-18
        .

        Args:
            page(int): page query parameter. Page number.
            size(int): size query parameter. Number of objects
                returned per page.
            filter(basestring, list, set, tuple): filter query
                parameter.               **Simple
                filtering** should be available through
                the filter query string parameter. The
                structure of a filter is a triplet of
                field operator and value separated with
                dots. More than one filter can be sent.
                The logical operator common to ALL
                filter criteria will be by default AND,
                and can be changed by using the
                "filterType=or" query string parameter.
                Each resource Data model description
                should specify if an attribute is a
                filtered field.              (Operator:
                Description),
                (EQ: Equals),               (NEQ: Not
                Equals),               (GT: Greater
                Than),               (LT: Less Then),
                (STARTSW: Starts With),
                (NSTARTSW: Not Starts With),
                (ENDSW: Ends With),
                (NENDSW: Not Ends With),
                (CONTAINS: Contains),
                (NCONTAINS: Not Contains),
                .
            filter_type(basestring): filterType query parameter. The
                logical operator common to ALL filter
                criteria will be by default AND, and can
                be changed by using the parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:
            Generator: A generator object containing the following object.

              + RestResponse: REST response with following properties:

                  - headers(MyDict): response headers.
                  - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                    or the bracket notation.
                  - content(bytes): representation of the request's response
                  - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """

        yield from get_next_page(
            self.get_node_details, dict(
                filter=filter,
                filter_type=filter_type,
                page=page,
                size=size,
                headers=headers,
                **query_parameters
            ),
            access_next_list=["SearchResult", "nextPage", "href"],
            access_resource_list=["SearchResult", "resources"])

    def get_all_generator(self,
                          filter=None,
                          filter_type=None,
                          page=None,
                          size=None,
                          headers=None,
                          **query_parameters):
        """Alias for `get_node_details_generator <#ciscoisesdk.
        api.v3_1_0.node_details.
        NodeDetails.get_node_details_generator>`_
        """
        yield from get_next_page(
            self.get_node_details, dict(
                filter=filter,
                filter_type=filter_type,
                page=page,
                size=size,
                headers=headers,
                **query_parameters
            ),
            access_next_list=["SearchResult", "nextPage", "href"],
            access_resource_list=["SearchResult", "resources"])

    def get_version(self,
                    headers=None,
                    **query_parameters):
        """This API helps to retrieve the version information realted to
        the node details.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/ers/config/node/versioninfo')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_ca9a3d8217d5507aa11020bee82ef228_v3_1_0', _api_response)



# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine NetworkDevice API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

from builtins import *

from past.builtins import basestring

from ...restsession import RestSession
from ...utils import (
    check_type,
    dict_from_items_with_values,
    apply_path_params,
    dict_of_str,
)
from ...pagination import get_next_page


class NetworkDevice(object):
    """Identity Services Engine NetworkDevice API (version: 3.1.1).

    Wraps the Identity Services Engine NetworkDevice
    API and exposes the API as native Python
    methods that return native Python objects.

    | Network Device API allows the client to add, delete, update, and search network devices. Please note that each API description shows whether the API is supported in bulk operation. The bulk section is showing only 'create' bulk operation however, all other operation which are bulk supported can be used in same way.

    **Revision History**

    +------------+------------------+---------------------+---------------------------------------------------------------------------------+-------------------------------------------------+-------------------------------------------------------------------------+
    | Revision # | Resource Version | Cisco   ISE Version | Description                                                                     | Revision   Modification                         | Revision   Modification                                                 |
    +------------+------------------+---------------------+---------------------------------------------------------------------------------+-------------------------------------------------+-------------------------------------------------------------------------+
    |            |                  |                     |                                                                                 | Attribute                                       | Description                                                             |
    +------------+------------------+---------------------+---------------------------------------------------------------------------------+-------------------------------------------------+-------------------------------------------------------------------------+
    | 0          | 1.0              | 1.2                 | Initial Cisco ISE Version                                                       |                                                 |                                                                         |
    +------------+------------------+---------------------+---------------------------------------------------------------------------------+-------------------------------------------------+-------------------------------------------------------------------------+
    | 1          | 1.1              | 2.0                 | Cisco ISE 2.0 model changes for TACACS+   and 3rd-Party support                 | tacacsSettings:sharedSecret                     | Added Attribute   'sharedSecret' for Tacacs+ support                    |
    +------------+------------------+---------------------+---------------------------------------------------------------------------------+-------------------------------------------------+-------------------------------------------------------------------------+
    |            |                  |                     |                                                                                 | tacacsSettings:connectModeOptions               | Added   Attribute 'connectModeOptions' for Tacacs+ support              |
    +------------+------------------+---------------------+---------------------------------------------------------------------------------+-------------------------------------------------+-------------------------------------------------------------------------+
    |            |                  |                     |                                                                                 | profileName                                     | Added   Attribute 'profileName' for 3rd-Party support support           |
    +------------+------------------+---------------------+---------------------------------------------------------------------------------+-------------------------------------------------+-------------------------------------------------------------------------+
    |            |                  |                     |                                                                                 | coaPort                                         | Added   Attribute 'coaPort' for 3rd-Party support support               |
    +------------+------------------+---------------------+---------------------------------------------------------------------------------+-------------------------------------------------+-------------------------------------------------------------------------+
    | 2          | 1.2              | 2.4                 | Cisco ISE 2.4 model changes for Multi shared secret   and DTLS Requierd support | authenticationsettings:enableMultiSecret        | Added   Attribute 'enableMultiSecret' for mulitple shared secret        |
    +------------+------------------+---------------------+---------------------------------------------------------------------------------+-------------------------------------------------+-------------------------------------------------------------------------+
    |            |                  |                     |                                                                                 | authenticationsettings:secondRadiusSharedSecret | Added   Attribute 'secondRadiusSharedSecret' for mulitple shared secret |
    +------------+------------------+---------------------+---------------------------------------------------------------------------------+-------------------------------------------------+-------------------------------------------------------------------------+
    |            |                  |                     |                                                                                 | authenticationsettings:dtlsRequired             | Added   Attribute 'dtlsRequired' for dtls settings                      |
    +------------+------------------+---------------------+---------------------------------------------------------------------------------+-------------------------------------------------+-------------------------------------------------------------------------+

    |

    **Resource Definition**

    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | **Attribute**                                    | **Type**  | **Required** | **Description**                                                                                        | **Default Values** | **Example Values**                                           |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | name                                             | String    | Yes          | Resource Name                                                                                          |                    | ISE_EST_Local_Host                                           |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | id                                               | String    | No           | Resource UUID, mandatory for update                                                                    |                    | 79aac430-7cc8-11eb-ad58-005056926583                         |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | description                                      | String    | No           |                                                                                                        |                    | example nd                                                   |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | authenticationSettings                           | List      | Yes          |                                                                                                        |                    |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - enableKeyWrap                                  | Boolean   | Yes          |                                                                                                        |                    | false                                                        |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - enabled                                        | Boolean   | Yes          |                                                                                                        | false              |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - keyEncryptionKey                               | String    | Yes          |                                                                                                        |                    | 1234567890123456                                             |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - keyInputFormat                                 | Enum      | Yes          | Allowed values: ASCII, HEXADECIMAL                                                                     | ASCII              |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - messageAuthenticatorCodeKey                    | String    | No           |                                                                                                        |                    | 12345678901234567890                                         |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - networkProtocol                                | Enum      | No           | Allowed values: RADIUS, TACACS_PLUS                                                                    | RADIUS             |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - radiusSharedSecret                             | String    | No           |                                                                                                        |                    | VHEGKOCCUHYB                                                 |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - enableMultiSecret                              | Boolean   | No           |                                                                                                        |                    | false                                                        |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - secondRadiusSharedSecret                       | String    | No           |                                                                                                        |                    |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - dtlsRequired                                   | Boolean   | No           | This value enforces use of dtls                                                                        | false              |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | coaPort                                          | Integer   | Yes          | Since 2.0 (for 3rd party)                                                                              |                    | 0                                                            |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | dtlsDnsName                                      | String    | No           | This value is used to verify the client identity contained in the X.509 RADIUS/DTLS client certificate |                    | ISE213.il.com                                                |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | NetworkDeviceIPList                              | List      | Yes          | List of IP Subnets for this node                                                                       |                    |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - ipaddress                                      | String    | Yes          | It can be either single IP address or IP range address                                                 |                    | 127.0.0.1                                                    |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - getIpaddressExclude                            | String    | Yes          | It can be either single IP address or IP range address                                                 |                    |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - mask                                           | Integer   | Yes          |                                                                                                        |                    | 32                                                           |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | NetworkDeviceGroupList                           | List      | No           | List of Network Device Group names for this node                                                       |                    | [ "Location#All Locations", "Device Type#All Device Types" ] |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | modelName                                        | String    | No           |                                                                                                        |                    |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | softwareVersion                                  | String    | No           |                                                                                                        |                    |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | profileName                                      | String    | Yes          | Since 2.0 (for 3rd party)                                                                              | true               | Cisco                                                        |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | snmpsettings                                     | List      | Yes          |                                                                                                        |                    |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - linkTrapQuery                                  | Boolean   | Yes          | SNMP link Trap Query                                                                                   | true               |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - macTrapQuery                                   | Boolean   | Yes          | SNMP MAC Trap Query                                                                                    | true               |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - originatingPolicyServicesNode                  | String    | Yes          | Originating Policy Services Node                                                                       | Auto               |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - pollingInterval                                | Integer   | Yes          | SNMP Polling Interval in seconds (Valid Range 600 to 86400)                                            | 3600               |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - roCommunity                                    | String    | No           | SNMP RO Community                                                                                      |                    | aaa                                                          |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - version                                        | Enum      | No           | Allowed values: ONE, TWO_C, THREE                                                                      |                    | ONE                                                          |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | tacacsSettings                                   | List      | Yes          | Since 2.0 (for Tacacs+)                                                                                |                    |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - connectModeOptions                             | Enum      | No           | Allowed values: OFF, ON_LEGACY, ON_DRAFT_COMPLIANT                                                     |                    | ON_LEGACY                                                    |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - sharedSecret                                   | String    | Yes          | Since 2.0                                                                                              |                    | aaa                                                          |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | trustsecsettings                                 | List      | Yes          |                                                                                                        |                    |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - deviceAuthenticationSettings                   | List      | Yes          |                                                                                                        |                    |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    |   - sgaDeviceId                                  | String    | No           |                                                                                                        |                    | networkDevice1                                               |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    |   - sgaDevicePassword                            | String    | No           |                                                                                                        |                    | aaa                                                          |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - deviceConfigurationDeployment                  | List      | Yes          |                                                                                                        |                    |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    |   - enableModePassword                           | String    | No           |                                                                                                        |                    | aaa                                                          |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    |   - execModePassword                             | String    | No           |                                                                                                        |                    | aaa                                                          |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    |   - execModeUsername                             | String    | No           |                                                                                                        |                    | aaa                                                          |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    |   - includeWhenDeployingSGTUpdates               | Boolean   | No           |                                                                                                        | false              |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    | - sgaNotificationAndUpdates                      | List      | Yes          |                                                                                                        |                    |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    |   - downlaodEnvironmentDataEveryXSeconds         | Integer   | No           |                                                                                                        | 86400              |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    |   - downlaodPeerAuthorizationPolicyEveryXSeconds | Integer   | No           |                                                                                                        | 86400              |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    |   - downloadSGACLListsEveryXSeconds              | Integer   | No           |                                                                                                        | 86400              |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    |   - otherSGADevicesToTrustThisDevice             | Boolean   | No           |                                                                                                        | false              |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    |   - reAuthenticationEveryXSeconds                | Integer   | No           |                                                                                                        | 86400              |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    |   - sendConfigurationToDevice                    | Boolean   | No           |                                                                                                        | false              |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+
    |   - sendConfigurationToDeviceUsing               | Enum      | No           | Allowed values: ENABLE_USING_COA, ENABLE_USING_CLI, DISABLE_ALL                                        | ENABLE_USING_COA   |                                                              |
    +--------------------------------------------------+-----------+--------------+--------------------------------------------------------------------------------------------------------+--------------------+--------------------------------------------------------------+

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new NetworkDevice
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(NetworkDevice, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def get_network_device_by_name(self,
                                   name,
                                   headers=None,
                                   **query_parameters):
        """This API allows the client to get a network device by name.

        Args:
            name(basestring): name path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           basestring)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(name, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'name': name,
        }

        e_url = ('/ers/config/networkdevice/name/{name}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_d8610d4a355b63aaaa364447d5fa00_v3_1_1', _api_response)

    def get_by_name(self,
                    name,
                    headers=None,
                    **query_parameters):
        """Alias for `get_network_device_by_name <#ciscoisesdk.
        api.v3_1_1.network_device.
        NetworkDevice.get_network_device_by_name>`_
        """
        return self.get_network_device_by_name(
            name=name,
            headers=headers,
            **query_parameters
        )

    def update_network_device_by_name(self,
                                      name,
                                      authentication_settings=None,
                                      coa_port=None,
                                      description=None,
                                      dtls_dns_name=None,
                                      id=None,
                                      model_name=None,
                                      network_device_group_list=None,
                                      network_device_iplist=None,
                                      profile_name=None,
                                      snmpsettings=None,
                                      software_version=None,
                                      tacacs_settings=None,
                                      trustsecsettings=None,
                                      headers=None,
                                      payload=None,
                                      active_validation=True,
                                      **query_parameters):
        """This API allows the client to update a network device by name.

        Args:
            network_device_group_list(list): List of Network Device
                Group names for this node, property of
                the request body (list of strings).
            network_device_iplist(list): List of IP Subnets for this
                node, property of the request body (list
                of objects).
            authentication_settings(object): authenticationSettings,
                property of the request body.
            coa_port(integer): coaPort, property of the request
                body.
            description(string): description, property of the
                request body.
            dtls_dns_name(string): This value is used to verify the
                client identity contained in the X.509
                RADIUS/DTLS client certificate, property
                of the request body.
            id(string): id, property of the request body.
            model_name(string): modelName, property of the request
                body.
            name(string): name, property of the request body.
            profile_name(string): profileName, property of the
                request body.
            snmpsettings(object): snmpsettings, property of the
                request body.
            software_version(string): softwareVersion, property of
                the request body.
            tacacs_settings(object): tacacsSettings, property of the
                request body.
            trustsecsettings(object): trustsecsettings, property of
                the request body.
            name(basestring): name path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           basestring)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(name, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'name': name,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'id':
                    id,
                'name':
                    name,
                'description':
                    description,
                'authenticationSettings':
                    authentication_settings,
                'snmpsettings':
                    snmpsettings,
                'trustsecsettings':
                    trustsecsettings,
                'tacacsSettings':
                    tacacs_settings,
                'profileName':
                    profile_name,
                'coaPort':
                    coa_port,
                'dtlsDnsName':
                    dtls_dns_name,
                'modelName':
                    model_name,
                'softwareVersion':
                    software_version,
                'NetworkDeviceIPList':
                    network_device_iplist,
                'NetworkDeviceGroupList':
                    network_device_group_list,
            }
            _payload = {
                'NetworkDevice': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_ea2c4586b845888b2a9375126f70de2_v3_1_1')\
                .validate(_payload)

        e_url = ('/ers/config/networkdevice/name/{name}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              **request_params)

        return self._object_factory('bpm_ea2c4586b845888b2a9375126f70de2_v3_1_1', _api_response)

    def update_by_name(self,
                       name,
                       authentication_settings=None,
                       coa_port=None,
                       description=None,
                       dtls_dns_name=None,
                       id=None,
                       model_name=None,
                       network_device_group_list=None,
                       network_device_iplist=None,
                       profile_name=None,
                       snmpsettings=None,
                       software_version=None,
                       tacacs_settings=None,
                       trustsecsettings=None,
                       headers=None,
                       payload=None,
                       active_validation=True,
                       **query_parameters):
        """Alias for `update_network_device_by_name <#ciscoisesdk.
        api.v3_1_1.network_device.
        NetworkDevice.update_network_device_by_name>`_
        """
        return self.update_network_device_by_name(
            name=name,
            authentication_settings=authentication_settings,
            coa_port=coa_port,
            description=description,
            dtls_dns_name=dtls_dns_name,
            id=id,
            model_name=model_name,
            network_device_group_list=network_device_group_list,
            network_device_iplist=network_device_iplist,
            profile_name=profile_name,
            snmpsettings=snmpsettings,
            software_version=software_version,
            tacacs_settings=tacacs_settings,
            trustsecsettings=trustsecsettings,
            payload=payload,
            active_validation=active_validation,
            headers=headers,
            **query_parameters
        )

    def delete_network_device_by_name(self,
                                      name,
                                      headers=None,
                                      **query_parameters):
        """This API deletes a network device by name.

        Args:
            name(basestring): name path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           basestring)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(name, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'name': name,
        }

        e_url = ('/ers/config/networkdevice/name/{name}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.delete(endpoint_full_url, params=_params,
                                                 headers=_headers)
        else:
            _api_response = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_eafaf2e785c6898fb982dbe4462e7_v3_1_1', _api_response)

    def delete_by_name(self,
                       name,
                       headers=None,
                       **query_parameters):
        """Alias for `delete_network_device_by_name <#ciscoisesdk.
        api.v3_1_1.network_device.
        NetworkDevice.delete_network_device_by_name>`_
        """
        return self.delete_network_device_by_name(
            name=name,
            headers=headers,
            **query_parameters
        )

    def get_network_device_by_id(self,
                                 id,
                                 headers=None,
                                 **query_parameters):
        """This API allows the client to get a network device by ID.

        Args:
            id(basestring): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           basestring)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(id, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        e_url = ('/ers/config/networkdevice/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_a4ab683ce53052e089626a096abaf451_v3_1_1', _api_response)

    def get_by_id(self,
                  id,
                  headers=None,
                  **query_parameters):
        """Alias for `get_network_device_by_id <#ciscoisesdk.
        api.v3_1_1.network_device.
        NetworkDevice.get_network_device_by_id>`_
        """
        return self.get_network_device_by_id(
            id=id,
            headers=headers,
            **query_parameters
        )

    def update_network_device_by_id(self,
                                    id,
                                    authentication_settings=None,
                                    coa_port=None,
                                    description=None,
                                    dtls_dns_name=None,
                                    model_name=None,
                                    name=None,
                                    network_device_group_list=None,
                                    network_device_iplist=None,
                                    profile_name=None,
                                    snmpsettings=None,
                                    software_version=None,
                                    tacacs_settings=None,
                                    trustsecsettings=None,
                                    headers=None,
                                    payload=None,
                                    active_validation=True,
                                    **query_parameters):
        """This API allows the client to update a network device by ID.

        Args:
            network_device_group_list(list): List of Network Device
                Group names for this node, property of
                the request body (list of strings).
            network_device_iplist(list): List of IP Subnets for this
                node, property of the request body (list
                of objects).
            authentication_settings(object): authenticationSettings,
                property of the request body.
            coa_port(integer): coaPort, property of the request
                body.
            description(string): description, property of the
                request body.
            dtls_dns_name(string): This value is used to verify the
                client identity contained in the X.509
                RADIUS/DTLS client certificate, property
                of the request body.
            id(string): id, property of the request body.
            model_name(string): modelName, property of the request
                body.
            name(string): name, property of the request body.
            profile_name(string): profileName, property of the
                request body.
            snmpsettings(object): snmpsettings, property of the
                request body.
            software_version(string): softwareVersion, property of
                the request body.
            tacacs_settings(object): tacacsSettings, property of the
                request body.
            trustsecsettings(object): trustsecsettings, property of
                the request body.
            id(basestring): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           basestring)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(id, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'id':
                    id,
                'name':
                    name,
                'description':
                    description,
                'authenticationSettings':
                    authentication_settings,
                'snmpsettings':
                    snmpsettings,
                'trustsecsettings':
                    trustsecsettings,
                'tacacsSettings':
                    tacacs_settings,
                'profileName':
                    profile_name,
                'coaPort':
                    coa_port,
                'dtlsDnsName':
                    dtls_dns_name,
                'modelName':
                    model_name,
                'softwareVersion':
                    software_version,
                'NetworkDeviceIPList':
                    network_device_iplist,
                'NetworkDeviceGroupList':
                    network_device_group_list,
            }
            _payload = {
                'NetworkDevice': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_b1edfeb182025176bb250633937177ae_v3_1_1')\
                .validate(_payload)

        e_url = ('/ers/config/networkdevice/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              **request_params)

        return self._object_factory('bpm_b1edfeb182025176bb250633937177ae_v3_1_1', _api_response)

    def update_by_id(self,
                     id,
                     authentication_settings=None,
                     coa_port=None,
                     description=None,
                     dtls_dns_name=None,
                     model_name=None,
                     name=None,
                     network_device_group_list=None,
                     network_device_iplist=None,
                     profile_name=None,
                     snmpsettings=None,
                     software_version=None,
                     tacacs_settings=None,
                     trustsecsettings=None,
                     headers=None,
                     payload=None,
                     active_validation=True,
                     **query_parameters):
        """Alias for `update_network_device_by_id <#ciscoisesdk.
        api.v3_1_1.network_device.
        NetworkDevice.update_network_device_by_id>`_
        """
        return self.update_network_device_by_id(
            id=id,
            authentication_settings=authentication_settings,
            coa_port=coa_port,
            description=description,
            dtls_dns_name=dtls_dns_name,
            model_name=model_name,
            name=name,
            network_device_group_list=network_device_group_list,
            network_device_iplist=network_device_iplist,
            profile_name=profile_name,
            snmpsettings=snmpsettings,
            software_version=software_version,
            tacacs_settings=tacacs_settings,
            trustsecsettings=trustsecsettings,
            payload=payload,
            active_validation=active_validation,
            headers=headers,
            **query_parameters
        )

    def delete_network_device_by_id(self,
                                    id,
                                    headers=None,
                                    **query_parameters):
        """This API deletes a network device by ID.

        Args:
            id(basestring): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           basestring)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(id, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        e_url = ('/ers/config/networkdevice/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.delete(endpoint_full_url, params=_params,
                                                 headers=_headers)
        else:
            _api_response = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_f2fd3c6324b581ca0f3f9eadede1cdc_v3_1_1', _api_response)

    def delete_by_id(self,
                     id,
                     headers=None,
                     **query_parameters):
        """Alias for `delete_network_device_by_id <#ciscoisesdk.
        api.v3_1_1.network_device.
        NetworkDevice.delete_network_device_by_id>`_
        """
        return self.delete_network_device_by_id(
            id=id,
            headers=headers,
            **query_parameters
        )

    def get_network_device(self,
                           filter=None,
                           filter_type=None,
                           page=None,
                           size=None,
                           sortasc=None,
                           sortdsc=None,
                           headers=None,
                           **query_parameters):
        """This API allows the client to get all the network devices.
        Filter:   [ipaddress, name, description, location, type]
        To search resources by using  toDate  column,follow the
        format:   DD-MON-YY (Example:13-SEP-18)     Day or
        Year:GET
        /ers/config/guestuser/?filter=toDate.CONTAINS.13
        Month:GET
        /ers/config/guestuser/?filter=toDate.CONTAINS.SEP
        Date:GET
        /ers/config/guestuser/?filter=toDate.CONTAINS.13-SEP-18
        Sorting:   [name, description].

        Args:
            page(int): page query parameter. Page number.
            size(int): size query parameter. Number of objects
                returned per page.
            sortasc(basestring): sortasc query parameter. sort asc.
            sortdsc(basestring): sortdsc query parameter. sort desc.
            filter(basestring, list, set, tuple): filter query
                parameter.               **Simple
                filtering** should be available through
                the filter query string parameter. The
                structure of a filter is a triplet of
                field operator and value separated with
                dots. More than one filter can be sent.
                The logical operator common to ALL
                filter criteria will be by default AND,
                and can be changed by using the
                "filterType=or" query string parameter.
                Each resource Data model description
                should specify if an attribute is a
                filtered field.              (Operator:
                Description),
                (EQ: Equals),               (NEQ: Not
                Equals),               (GT: Greater
                Than),               (LT: Less Then),
                (STARTSW: Starts With),
                (NSTARTSW: Not Starts With),
                (ENDSW: Ends With),
                (NENDSW: Not Ends With),
                (CONTAINS: Contains),
                (NCONTAINS: Not Contains),
                .
            filter_type(basestring): filterType query parameter. The
                logical operator common to ALL filter
                criteria will be by default AND, and can
                be changed by using the parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           basestring)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(page, (int, basestring, list))
        check_type(size, (int, basestring, list))
        check_type(sortasc, basestring)
        check_type(sortdsc, basestring)
        check_type(filter, (basestring, list, set, tuple))
        check_type(filter_type, basestring)

        _params = {
            'page':
                page,
            'size':
                size,
            'sortasc':
                sortasc,
            'sortdsc':
                sortdsc,
            'filter':
                filter,
            'filterType':
                filter_type,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/ers/config/networkdevice')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_b986fa0f0d54ef98eb135eeb88808a_v3_1_1', _api_response)

    def get_all(self,
                filter=None,
                filter_type=None,
                page=None,
                size=None,
                sortasc=None,
                sortdsc=None,
                headers=None,
                **query_parameters):
        """Alias for `get_network_device <#ciscoisesdk.
        api.v3_1_1.network_device.
        NetworkDevice.get_network_device>`_
        """
        return self.get_network_device(
            filter=filter,
            filter_type=filter_type,
            page=page,
            size=size,
            sortasc=sortasc,
            sortdsc=sortdsc,
            headers=headers,
            **query_parameters
        )

    def get_network_device_generator(self,
                                     filter=None,
                                     filter_type=None,
                                     page=None,
                                     size=None,
                                     sortasc=None,
                                     sortdsc=None,
                                     headers=None,
                                     **query_parameters):
        """This API allows the client to get all the network devices.
        Filter:   [ipaddress, name, description, location, type]
        To search resources by using  toDate  column,follow the
        format:   DD-MON-YY (Example:13-SEP-18)     Day or
        Year:GET
        /ers/config/guestuser/?filter=toDate.CONTAINS.13
        Month:GET
        /ers/config/guestuser/?filter=toDate.CONTAINS.SEP
        Date:GET
        /ers/config/guestuser/?filter=toDate.CONTAINS.13-SEP-18
        Sorting:   [name, description].

        Args:
            page(int): page query parameter. Page number.
            size(int): size query parameter. Number of objects
                returned per page.
            sortasc(basestring): sortasc query parameter. sort asc.
            sortdsc(basestring): sortdsc query parameter. sort desc.
            filter(basestring, list, set, tuple): filter query
                parameter.               **Simple
                filtering** should be available through
                the filter query string parameter. The
                structure of a filter is a triplet of
                field operator and value separated with
                dots. More than one filter can be sent.
                The logical operator common to ALL
                filter criteria will be by default AND,
                and can be changed by using the
                "filterType=or" query string parameter.
                Each resource Data model description
                should specify if an attribute is a
                filtered field.              (Operator:
                Description),
                (EQ: Equals),               (NEQ: Not
                Equals),               (GT: Greater
                Than),               (LT: Less Then),
                (STARTSW: Starts With),
                (NSTARTSW: Not Starts With),
                (ENDSW: Ends With),
                (NENDSW: Not Ends With),
                (CONTAINS: Contains),
                (NCONTAINS: Not Contains),
                .
            filter_type(basestring): filterType query parameter. The
                logical operator common to ALL filter
                criteria will be by default AND, and can
                be changed by using the parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:
            Generator: A generator object containing the following object.

              + RestResponse: REST response with following properties:

                  - headers(MyDict): response headers.
                  - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                    or the bracket notation.
                  - content(bytes): representation of the request's response
                  - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """

        yield from get_next_page(
            self.get_network_device, dict(
                filter=filter,
                filter_type=filter_type,
                page=page,
                size=size,
                sortasc=sortasc,
                sortdsc=sortdsc,
                headers=headers,
                **query_parameters
            ),
            access_next_list=["SearchResult", "nextPage", "href"],
            access_resource_list=["SearchResult", "resources"])

    def get_all_generator(self,
                          filter=None,
                          filter_type=None,
                          page=None,
                          size=None,
                          sortasc=None,
                          sortdsc=None,
                          headers=None,
                          **query_parameters):
        """Alias for `get_network_device_generator <#ciscoisesdk.
        api.v3_1_1.network_device.
        NetworkDevice.get_network_device_generator>`_
        """
        yield from get_next_page(
            self.get_network_device, dict(
                filter=filter,
                filter_type=filter_type,
                page=page,
                size=size,
                sortasc=sortasc,
                sortdsc=sortdsc,
                headers=headers,
                **query_parameters
            ),
            access_next_list=["SearchResult", "nextPage", "href"],
            access_resource_list=["SearchResult", "resources"])

    def create_network_device(self,
                              authentication_settings=None,
                              coa_port=None,
                              description=None,
                              dtls_dns_name=None,
                              model_name=None,
                              name=None,
                              network_device_group_list=None,
                              network_device_iplist=None,
                              profile_name=None,
                              snmpsettings=None,
                              software_version=None,
                              tacacs_settings=None,
                              trustsecsettings=None,
                              headers=None,
                              payload=None,
                              active_validation=True,
                              **query_parameters):
        """This API creates a network device.

        Args:
            network_device_group_list(list): List of Network Device
                Group names for this node, property of
                the request body (list of strings).
            network_device_iplist(list): List of IP Subnets for this
                node, property of the request body (list
                of objects).
            authentication_settings(object): authenticationSettings,
                property of the request body.
            coa_port(integer): coaPort, property of the request
                body.
            description(string): description, property of the
                request body.
            dtls_dns_name(string): This value is used to verify the
                client identity contained in the X.509
                RADIUS/DTLS client certificate, property
                of the request body.
            model_name(string): modelName, property of the request
                body.
            name(string): name, property of the request body.
            profile_name(string): profileName, property of the
                request body.
            snmpsettings(object): snmpsettings, property of the
                request body.
            software_version(string): softwareVersion, property of
                the request body.
            tacacs_settings(object): tacacsSettings, property of the
                request body.
            trustsecsettings(object): trustsecsettings, property of
                the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           basestring)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'name':
                    name,
                'description':
                    description,
                'authenticationSettings':
                    authentication_settings,
                'snmpsettings':
                    snmpsettings,
                'trustsecsettings':
                    trustsecsettings,
                'tacacsSettings':
                    tacacs_settings,
                'profileName':
                    profile_name,
                'coaPort':
                    coa_port,
                'dtlsDnsName':
                    dtls_dns_name,
                'modelName':
                    model_name,
                'softwareVersion':
                    software_version,
                'NetworkDeviceIPList':
                    network_device_iplist,
                'NetworkDeviceGroupList':
                    network_device_group_list,
            }
            _payload = {
                'NetworkDevice': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_ca6ab8ec556c3bc9531dc380b230a_v3_1_1')\
                .validate(_payload)

        e_url = ('/ers/config/networkdevice')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_ca6ab8ec556c3bc9531dc380b230a_v3_1_1', _api_response)

    def create(self,
               authentication_settings=None,
               coa_port=None,
               description=None,
               dtls_dns_name=None,
               model_name=None,
               name=None,
               network_device_group_list=None,
               network_device_iplist=None,
               profile_name=None,
               snmpsettings=None,
               software_version=None,
               tacacs_settings=None,
               trustsecsettings=None,
               headers=None,
               payload=None,
               active_validation=True,
               **query_parameters):
        """Alias for `create_network_device <#ciscoisesdk.
        api.v3_1_1.network_device.
        NetworkDevice.create_network_device>`_
        """
        return self.create_network_device(
            authentication_settings=authentication_settings,
            coa_port=coa_port,
            description=description,
            dtls_dns_name=dtls_dns_name,
            model_name=model_name,
            name=name,
            network_device_group_list=network_device_group_list,
            network_device_iplist=network_device_iplist,
            profile_name=profile_name,
            snmpsettings=snmpsettings,
            software_version=software_version,
            tacacs_settings=tacacs_settings,
            trustsecsettings=trustsecsettings,
            payload=payload,
            active_validation=active_validation,
            headers=headers,
            **query_parameters
        )

    def get_version(self,
                    headers=None,
                    **query_parameters):
        """This API helps to retrieve the version information related to
        the network device.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/ers/config/networkdevice/versioninfo')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_e571185718b6ef6e78bfbfdf68_v3_1_1', _api_response)

    def bulk_request_for_network_device(self,
                                        operation_type=None,
                                        resource_media_type=None,
                                        headers=None,
                                        payload=None,
                                        active_validation=True,
                                        **query_parameters):
        """This API allows the client to submit the bulk request.

        Args:
            operation_type(string): operationType, property of the
                request body.
            resource_media_type(string): resourceMediaType, property
                of the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'operationType':
                    operation_type,
                'resourceMediaType':
                    resource_media_type,
            }
            _payload = {
                'NetworkDeviceBulkRequest': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_b2eebd5c245e58a503aa53115eec53_v3_1_1')\
                .validate(_payload)

        e_url = ('/ers/config/networkdevice/bulk/submit')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              **request_params)

        return self._object_factory('bpm_b2eebd5c245e58a503aa53115eec53_v3_1_1', _api_response)

    def bulk_request(self,
                     operation_type=None,
                     resource_media_type=None,
                     headers=None,
                     payload=None,
                     active_validation=True,
                     **query_parameters):
        """Alias for `bulk_request_for_network_device <#ciscoisesdk.
        api.v3_1_1.network_device.
        NetworkDevice.bulk_request_for_network_device>`_
        """
        return self.bulk_request_for_network_device(
            operation_type=operation_type,
            resource_media_type=resource_media_type,
            payload=payload,
            active_validation=active_validation,
            headers=headers,
            **query_parameters
        )

    def monitor_bulk_status_network_device(self,
                                           bulkid,
                                           headers=None,
                                           **query_parameters):
        """This API allows the client to monitor the bulk request.

        Args:
            bulkid(basestring): bulkid path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(bulkid, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'bulkid': bulkid,
        }

        e_url = ('/ers/config/networkdevice/bulk/{bulkid}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_bf96800cc265b5e9e1566ec7088619c_v3_1_1', _api_response)

    def monitor_bulk_status(self,
                            bulkid,
                            headers=None,
                            **query_parameters):
        """Alias for `monitor_bulk_status_network_device <#ciscoisesdk.
        api.v3_1_1.network_device.
        NetworkDevice.monitor_bulk_status_network_device>`_
        """
        return self.monitor_bulk_status_network_device(
            bulkid=bulkid,
            headers=headers,
            **query_parameters
        )



# -*- coding: utf-8 -*-
"""Identity Services Engine Authentication API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

from future import standard_library
standard_library.install_aliases()

from builtins import *
import urllib.parse

from past.builtins import basestring
import requests

from ..response_codes import EXPECTED_RESPONSE_CODE
from ..utils import (
    check_type,
    dict_from_items_with_values,
    extract_and_parse_json,
    validate_base_url,
)
from ..misc import (
    check_response_code,
)
from base64 import b64encode


class Authentication(object):
    """Identity Services Engine Authentication API.

    Wraps the Identity Services Engine Authentication API and exposes the API as native
    Python methods that return native Python objects.

    """

    def __init__(self, base_url, object_factory, single_request_timeout=None,
                 verify=True):
        """Initialize an Authentication
        object with the provided RestSession.

        Args:
            base_url(basestring): The base URL to be prefixed to the
                individual API endpoint suffixes.
            object_factory(callable): The factory function to use to create
                Python objects from the returned Identity Services Engine JSON data objects.
            single_request_timeout(int): Timeout in seconds for the API
                requests.
            verify(bool,basestring): Controls whether we verify the server's
                TLS certificate, or a string, in which case it must be a path
                to a CA bundle to use.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(base_url, basestring, may_be_none=False)
        check_type(single_request_timeout, int)
        check_type(verify, (bool, basestring), may_be_none=False)

        super(Authentication, self).__init__()

        self._base_url = str(validate_base_url(base_url))
        self._single_request_timeout = single_request_timeout
        self._verify = verify
        self._request_kwargs = {"timeout": single_request_timeout,
                                "verify": verify}
        self._object_factory = object_factory

    @property
    def verify(self):
        """The verify (TLS Certificate) for the API endpoints."""
        return self._verify

    @property
    def base_url(self):
        """The base URL for the API endpoints."""
        return self._base_url

    @property
    def single_request_timeout(self):
        """Timeout in seconds for the API requests."""
        return self._single_request_timeout

    @verify.setter
    def verify(self, value):
        """The verify (TLS Certificate) for the API endpoints."""
        check_type(value, (bool, basestring), may_be_none=False)
        self._verify = value
        self._request_kwargs = {"timeout": self._single_request_timeout,
                                "verify": self._verify}

    @base_url.setter
    def base_url(self, value):
        """The base URL for the API endpoints."""
        check_type(value, basestring, may_be_none=False)
        self._base_url = str(validate_base_url(value))

    @single_request_timeout.setter
    def single_request_timeout(self, value):
        """The timeout (seconds) for a single HTTP REST API request."""
        check_type(value, int)
        assert value is None or value > 0
        self._single_request_timeout = value
        self._request_kwargs = {"timeout": self._single_request_timeout,
                                "verify": self._verify}

    def authentication_api(self, username, password, encoded_auth=None):
        """Exchange basic auth data for a Authorization Basic encoded value
        that can be used to invoke the APIs.

        Args:
            username(basestring): HTTP Basic Auth username.
            password(basestring): HTTP Basic Auth password.
            encoded_auth(basestring): HTTP Basic Auth base64 encoded string.

        Returns:
            AccessToken: An AccessToken object with the access token provided
            by the Identity Services Engine cloud.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        if encoded_auth is not None:
            check_type(encoded_auth, basestring, may_be_none=False)
            if isinstance(encoded_auth, str):
                encoded_auth = bytes(encoded_auth, 'utf-8')
        else:
            check_type(username, basestring, may_be_none=False)
            check_type(password, basestring, may_be_none=False)
            encoded_auth = b64encode(bytes(username + ':' + password, "utf-8"))

        if isinstance(encoded_auth, bytes):
            encoded_auth = encoded_auth.decode('utf-8')

        json_data = {'Token': encoded_auth}
        return self._object_factory('bpm_ac8ae94c4e69a09d', json_data)



# -*- coding: utf-8 -*-
"""Functions that deal with ISE pagination.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

from future import standard_library
standard_library.install_aliases()
native_str = str
import urllib.parse
from .exceptions import ApiError


def get_next_page_number(page_param):
    next_page_number = 0
    if isinstance(page_param, list):
        if len(page_param) > 0:
            page_param = page_param[0]
    if isinstance(page_param, str):
        try:
            next_page_number = int(page_param)
        except ValueError:
            next_page_number = 0
    elif isinstance(page_param, int):
        next_page_number = page_param
    next_page_number = (next_page_number or 1) + 1
    return next_page_number


def get_next_page(function, params, access_next_list=["SearchResult", "nextPage", "href"],
                  access_resource_list=["SearchResult", "resources"],
                  has_been_found=None, prev_result=None):
    """
    Args:
        function(function): The API function to call
        params(dict): The parameters of the function
        access_next_list(list): List of strings. Allows to access the URL for the next page using the previous response object
        access_resource_list(list): List of strings. Allows to access the response object

    Yields:
        Generator: A generator object containing the RestResponse objects for all pages.

    """
    response = function(**params)
    if response.response:
        result = response.response
        value = response.response
        items_found = 0
        for access_next in access_next_list:
            if isinstance(value, dict) and value.get(access_next) is not None:
                value = value.get(access_next)
                items_found = items_found + 1
            else:
                items_found = 0
                break
        found = isinstance(value, str) and len(access_next_list) == items_found
        if has_been_found is None:
            has_been_found = found

        for access_resource in access_resource_list:
            if isinstance(result, dict) and result.get(access_resource) is not None:
                result = result.get(access_resource)

        if prev_result != result:
            prev_result = result
            if not found:
                if isinstance(result, list) and not has_been_found:
                    if len(result) == 0:
                        yield response
                    else:
                        yield response
                        _params = dict(params)
                        if 'page' in params and 'size' in params:
                            _params['page'] = get_next_page_number(params['page'])
                        try:
                            yield from get_next_page(
                                function, {**_params},
                                access_next_list=access_next_list, access_resource_list=access_resource_list,
                                has_been_found=has_been_found, prev_result=prev_result)
                        except ApiError as e:
                            expected_status_codes = [400, 404]
                            # If is not a 400 Bad Request and is not a 404 Not Found status code
                            if e.status_code not in expected_status_codes:
                                raise e
                else:
                    yield response
            else:
                yield response
                url = value
                _query_params = urllib.parse.parse_qs(urllib.parse.urlparse(url).query)
                yield from get_next_page(
                    function, {**params, **_query_params},
                    access_next_list=access_next_list, access_resource_list=access_resource_list,
                    has_been_found=has_been_found, prev_result=prev_result)



# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine Backup And Restore API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

from builtins import *

from past.builtins import basestring

from ...restsession import RestSession
from ...utils import (
    check_type,
    dict_from_items_with_values,
    apply_path_params,
    dict_of_str,
)
from ...pagination import get_next_page


class BackupAndRestore(object):
    """Identity Services Engine Backup And Restore API (version: 3.1.0).

    Wraps the Identity Services Engine Backup And Restore
    API and exposes the API as native Python
    methods that return native Python objects.



    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new BackupAndRestore
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(BackupAndRestore, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def config_backup(self,
                      backup_encryption_key=None,
                      backup_name=None,
                      repository_name=None,
                      headers=None,
                      payload=None,
                      active_validation=True,
                      **query_parameters):
        """Triggers on demand configuration backup on the ISE node. The API
        returns the task ID. Use the Task Service status API to
        get the status of the backup job.

        Args:
            backup_encryption_key(string): The encyption key for the
                backed up file. Encryption key must
                satisfy the following criteria Contains
                at least one uppercase letter [A-Z],
                Contains at least one lowercase letter
                [a-z], Contains at least one digit
                [0-9], Contain only [A-Z][a-z][0-9]_#,
                Has at least 8 characters, Has not more
                than 15 characters, Must not contain
                'CcIiSsCco', Must not begin with,
                property of the request body.
            backup_name(string): The backup file will get saved with
                this name., property of the request
                body.
            repository_name(string): Name of the configured
                repository where the generated backup
                file will get copied., property of the
                request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'backupEncryptionKey':
                    backup_encryption_key,
                'backupName':
                    backup_name,
                'repositoryName':
                    repository_name,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_db1d9dda53369e35d33138b29c16_v3_1_0')\
                .validate(_payload)

        e_url = ('/api/v1/backup-restore/config/backup')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_db1d9dda53369e35d33138b29c16_v3_1_0', _api_response)

    def config(self,
               backup_encryption_key=None,
               backup_name=None,
               repository_name=None,
               headers=None,
               payload=None,
               active_validation=True,
               **query_parameters):
        """Alias for `config_backup <#ciscoisesdk.
        api.v3_1_0.backup_and_restore.
        BackupAndRestore.config_backup>`_
        """
        return self.config_backup(
            backup_encryption_key=backup_encryption_key,
            backup_name=backup_name,
            repository_name=repository_name,
            payload=payload,
            active_validation=active_validation,
            headers=headers,
            **query_parameters
        )

    def cancel_backup(self,
                      headers=None,
                      **query_parameters):
        """Cancels the backup job running on the node.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/api/v1/backup-restore/config/cancel-backup')
        endpoint_full_url = apply_path_params(e_url, path_params)

        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params)

        return self._object_factory('bpm_e155669bc74586e9ef2580ec5752902_v3_1_0', _api_response)

    def cancel(self,
               headers=None,
               **query_parameters):
        """Alias for `cancel_backup <#ciscoisesdk.
        api.v3_1_0.backup_and_restore.
        BackupAndRestore.cancel_backup>`_
        """
        return self.cancel_backup(
            headers=headers,
            **query_parameters
        )

    def get_last_config_backup_status(self,
                                      headers=None,
                                      **query_parameters):
        """Gives the last backup status.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/api/v1/backup-restore/config/last-backup-status')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_d388e26255a15233ac682c0406880cfb_v3_1_0', _api_response)

    def get_last_status(self,
                        headers=None,
                        **query_parameters):
        """Alias for `get_last_config_backup_status <#ciscoisesdk.
        api.v3_1_0.backup_and_restore.
        BackupAndRestore.get_last_config_backup_status>`_
        """
        return self.get_last_config_backup_status(
            headers=headers,
            **query_parameters
        )

    def restore_config_backup(self,
                              backup_encryption_key=None,
                              repository_name=None,
                              restore_file=None,
                              restore_include_adeos=None,
                              headers=None,
                              payload=None,
                              active_validation=True,
                              **query_parameters):
        """Triggers a configuration DB restore job on the ISE node. The API
        returns the task ID. Use the Task Service status API to
        get the status of the backup job.

        Args:
            backup_encryption_key(string): The encryption key which
                was provided at the time of taking
                backup., property of the request body.
            repository_name(string): Name of the configred
                repository where the backup file
                exists., property of the request body.
            restore_file(string): Name of the backup file to be
                restored on ISE node., property of the
                request body.
            restore_include_adeos(string): Determines whether the
                ADE-OS configure is restored. Possible
                values true, false, property of the
                request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'backupEncryptionKey':
                    backup_encryption_key,
                'repositoryName':
                    repository_name,
                'restoreFile':
                    restore_file,
                'restoreIncludeAdeos':
                    restore_include_adeos,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_b8319a8b5d195348a8763acd95ca2967_v3_1_0')\
                .validate(_payload)

        e_url = ('/api/v1/backup-restore/config/restore')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_b8319a8b5d195348a8763acd95ca2967_v3_1_0', _api_response)

    def restore(self,
                backup_encryption_key=None,
                repository_name=None,
                restore_file=None,
                restore_include_adeos=None,
                headers=None,
                payload=None,
                active_validation=True,
                **query_parameters):
        """Alias for `restore_config_backup <#ciscoisesdk.
        api.v3_1_0.backup_and_restore.
        BackupAndRestore.restore_config_backup>`_
        """
        return self.restore_config_backup(
            backup_encryption_key=backup_encryption_key,
            repository_name=repository_name,
            restore_file=restore_file,
            restore_include_adeos=restore_include_adeos,
            payload=payload,
            active_validation=active_validation,
            headers=headers,
            **query_parameters
        )

    def update_scheduled_config_backup(self,
                                       backup_description=None,
                                       backup_encryption_key=None,
                                       backup_name=None,
                                       end_date=None,
                                       frequency=None,
                                       month_day=None,
                                       repository_name=None,
                                       start_date=None,
                                       status=None,
                                       time=None,
                                       week_day=None,
                                       headers=None,
                                       payload=None,
                                       active_validation=True,
                                       **query_parameters):
        """Update the Schedule of the configuration backup on the ISE node
        as per the input parameters. This API only helps in
        editing the schedule.

        Args:
            backup_description(string): Description of the backup.,
                property of the request body.
            backup_encryption_key(string): The encyption key for the
                backed up file. Encryption key must
                satisfy the following criteria Contains
                at least one uppercase letter [A-Z],
                Contains at least one lowercase letter
                [a-z], Contains at least one digit
                [0-9], Contain only [A-Z][a-z][0-9]_#,
                Has at least 8 characters, Has not more
                than 15 characters, Must not contain
                'CcIiSsCco', Must not begin with,
                property of the request body.
            backup_name(string): The backup file will get saved with
                this name., property of the request
                body.
            end_date(string): End date of the scheduled backup job.
                Allowed format MM/DD/YYYY. End date is
                not required in case of ONE_TIME
                frequency., property of the request
                body.
            frequency(string): Frequency with which the backup will
                get scheduled in the ISE node., property
                of the request body. Available values
                are 'DAILY', 'MONTHLY', 'ONCE' and
                'WEEKLY'.
            month_day(string): Day of month you want backup to be
                performed on when scheduled frequency is
                MONTHLY. Allowed values from 1 to 28.,
                property of the request body.
            repository_name(string): Name of the configured
                repository where the generated backup
                file will get copied., property of the
                request body.
            start_date(string): Start date for scheduling the backup
                job. Allowed format MM/DD/YYYY.,
                property of the request body.
            status(string): Enable or disable scheduled backup.,
                property of the request body. Available
                values are 'DISABLE' and 'ENABLE'.
            time(string): Time at which backup job get scheduled.
                example12:00 AM, property of the request
                body.
            week_day(string): Day of week you want backup to be
                performed on when scheduled frequency is
                WEEKLY, property of the request body.
                Available values are 'FRI', 'MON',
                'SAT', 'SUN', 'THU', 'TUE' and 'WED'.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'backupDescription':
                    backup_description,
                'backupEncryptionKey':
                    backup_encryption_key,
                'backupName':
                    backup_name,
                'endDate':
                    end_date,
                'frequency':
                    frequency,
                'monthDay':
                    month_day,
                'repositoryName':
                    repository_name,
                'startDate':
                    start_date,
                'status':
                    status,
                'time':
                    time,
                'weekDay':
                    week_day,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_fc7103b05336a7960d9f34033eca_v3_1_0')\
                .validate(_payload)

        e_url = ('/api/v1/backup-restore/config/schedule-config-backup')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              **request_params)

        return self._object_factory('bpm_fc7103b05336a7960d9f34033eca_v3_1_0', _api_response)

    def update(self,
               backup_description=None,
               backup_encryption_key=None,
               backup_name=None,
               end_date=None,
               frequency=None,
               month_day=None,
               repository_name=None,
               start_date=None,
               status=None,
               time=None,
               week_day=None,
               headers=None,
               payload=None,
               active_validation=True,
               **query_parameters):
        """Alias for `update_scheduled_config_backup <#ciscoisesdk.
        api.v3_1_0.backup_and_restore.
        BackupAndRestore.update_scheduled_config_backup>`_
        """
        return self.update_scheduled_config_backup(
            backup_description=backup_description,
            backup_encryption_key=backup_encryption_key,
            backup_name=backup_name,
            end_date=end_date,
            frequency=frequency,
            month_day=month_day,
            repository_name=repository_name,
            start_date=start_date,
            status=status,
            time=time,
            week_day=week_day,
            payload=payload,
            active_validation=active_validation,
            headers=headers,
            **query_parameters
        )

    def create_scheduled_config_backup(self,
                                       backup_description=None,
                                       backup_encryption_key=None,
                                       backup_name=None,
                                       end_date=None,
                                       frequency=None,
                                       month_day=None,
                                       repository_name=None,
                                       start_date=None,
                                       status=None,
                                       time=None,
                                       week_day=None,
                                       headers=None,
                                       payload=None,
                                       active_validation=True,
                                       **query_parameters):
        """Schedules the configuration backup on the ISE node as per the
        input parameters. This API helps in creating the
        schedule for the first time.

        Args:
            backup_description(string): Description of the backup.,
                property of the request body.
            backup_encryption_key(string): The encyption key for the
                backed up file. Encryption key must
                satisfy the following criteria Contains
                at least one uppercase letter [A-Z],
                Contains at least one lowercase letter
                [a-z], Contains at least one digit
                [0-9], Contain only [A-Z][a-z][0-9]_#,
                Has at least 8 characters, Has not more
                than 15 characters, Must not contain
                'CcIiSsCco', Must not begin with,
                property of the request body.
            backup_name(string): The backup file will get saved with
                this name., property of the request
                body.
            end_date(string): End date of the scheduled backup job.
                Allowed format MM/DD/YYYY. End date is
                not required in case of ONE_TIME
                frequency., property of the request
                body.
            frequency(string): Frequency with which the backup will
                get scheduled in the ISE node., property
                of the request body. Available values
                are 'DAILY', 'MONTHLY', 'ONCE' and
                'WEEKLY'.
            month_day(string): Day of month you want backup to be
                performed on when scheduled frequency is
                MONTHLY. Allowed values from 1 to 28.,
                property of the request body.
            repository_name(string): Name of the configured
                repository where the generated backup
                file will get copied., property of the
                request body.
            start_date(string): Start date for scheduling the backup
                job. Allowed format MM/DD/YYYY.,
                property of the request body.
            status(string): Enable or disable scheduled backup.,
                property of the request body. Available
                values are 'DISABLE' and 'ENABLE'.
            time(string): Time at which backup job get scheduled.
                example12:00 AM, property of the request
                body.
            week_day(string): Day of week you want backup to be
                performed on when scheduled frequency is
                WEEKLY, property of the request body.
                Available values are 'FRI', 'MON',
                'SAT', 'SUN', 'THU', 'TUE' and 'WED'.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'backupDescription':
                    backup_description,
                'backupEncryptionKey':
                    backup_encryption_key,
                'backupName':
                    backup_name,
                'endDate':
                    end_date,
                'frequency':
                    frequency,
                'monthDay':
                    month_day,
                'repositoryName':
                    repository_name,
                'startDate':
                    start_date,
                'status':
                    status,
                'time':
                    time,
                'weekDay':
                    week_day,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_b994e6c8b8d53f29230686824c9fafa_v3_1_0')\
                .validate(_payload)

        e_url = ('/api/v1/backup-restore/config/schedule-config-backup')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_b994e6c8b8d53f29230686824c9fafa_v3_1_0', _api_response)

    def create(self,
               backup_description=None,
               backup_encryption_key=None,
               backup_name=None,
               end_date=None,
               frequency=None,
               month_day=None,
               repository_name=None,
               start_date=None,
               status=None,
               time=None,
               week_day=None,
               headers=None,
               payload=None,
               active_validation=True,
               **query_parameters):
        """Alias for `create_scheduled_config_backup <#ciscoisesdk.
        api.v3_1_0.backup_and_restore.
        BackupAndRestore.create_scheduled_config_backup>`_
        """
        return self.create_scheduled_config_backup(
            backup_description=backup_description,
            backup_encryption_key=backup_encryption_key,
            backup_name=backup_name,
            end_date=end_date,
            frequency=frequency,
            month_day=month_day,
            repository_name=repository_name,
            start_date=start_date,
            status=status,
            time=time,
            week_day=week_day,
            payload=payload,
            active_validation=active_validation,
            headers=headers,
            **query_parameters
        )
